"""Climpact plugin, that processes climate model output to impact model input."""
import json
import logging
import os
import sys
from getpass import getuser
from pathlib import Path
from subprocess import check_call
from tempfile import NamedTemporaryFile
from typing import Any, Dict, List

import freva
from climpact_assets import MUNICIPALITIES
from evaluation_system.api import plugin
from evaluation_system.api.parameters import (
    InputDirectory,
)
from evaluation_system.api.parameters import ParameterDictionary as ParamDict
from evaluation_system.api.parameters import (
    SelectField,
    SolrField,
    String,
)
from evaluation_system.misc import config
from evaluation_system.model.solr import SolrFindFiles


def get_vault_url(_url: List[str] = []) -> str:
    """Construct the url to read the db secrets."""
    if _url:
        return _url[0]
    try:
        host = config.get("db.host")
    except TypeError:
        config.reloadConfiguration()
        host = config.get("db.host")
    key = config._get_public_key(config.get("project_name"))
    _url.append(f"http://{host}:5002/vault/data/{key}")
    return _url[0]


def read_file_patters(_config: Dict[str, List[str]] = {}) -> Dict[str, str]:
    """Read the file patterns from the mongodb."""
    if _config:
        return {f: f for f in _config["file_pattern"]}
    with NamedTemporaryFile(suffix=".json") as tempf:
        import zonk_assets

        zonk_dir = Path(zonk_assets.__file__).parent
        cmd = [
            str(zonk_dir.parent / "plugin_env" / "bin" / "python"),
            str(zonk_dir / "read_db.py"),
            get_vault_url(),
            tempf.name,
        ]
        check_call(cmd)
        _config = json.loads(Path(tempf.name).read_text())
    return {f: f for f in _config["file_pattern"]}


class Zonk(plugin.PluginAbstract):
    """Zonk - Zarr Over Netcdf with Kerchunk."""

    tool_developer = {"name": "Martin Bergemann", "email": "bergemann@dkrz.de"}
    __category__ = "support"
    __short_description__ = (
        "Access your experiment data via virtual zarr datasets."
    )
    __long_description__ = (
        "ZONK - Zarr Over Netcdf with Kerchunk - is a freva plugin that "
        "generates Kerchunk reference files from NetCDF datasets, enabling "
        "efficient, virtual access to scientific data as Zarr without data "
        "duplication."
    )
    __version__ = (2502, 0, 0)
    __parameters__ = ParamDict(
        InputDirectory(
            name="input_directory",
            help=(
                "Select the path to your data. You can leave this empty if "
                "you whish to make the data selection based on experiment meta"
                "data with help of the databrowser."
            ),
        ),
        SolrField(
            name="project",
            facet="product",
            help=(
                "Set the project of the input data. Will only have an "
                "effect if you haven't selected the <input_directory> "
                "parameter."
            ),
        ),
        SolrField(
            name="product",
            facet="product",
            help=(
                "Set the project of the input data. Will only have an "
                "effect if you haven't selected the <input_directory> "
                "parameter."
            ),
        ),
        SolrField(
            name="model",
            facet="model",
            help=(
                "Set the model name that created the input data. Will only "
                "have and effect if you haven't selected the "
                "<input_directory> parameter."
            ),
        ),
        SolrField(
            name="experiment",
            facet="experiment",
            multiple=True,
            help=(
                "Set the experiment(s) of the input data. Will only have an "
                "effect if you haven't selected the <input_directory> "
                "parameter."
            ),
        ),
        SolrField(
            name="variable",
            facet="variable",
            multiple=True,
            help=(
                "Set the variable(s) of the input data. Will only have an "
                "effect if you haven't selected the <input_directory> "
                "parameter."
            ),
        ),
        SolrField(
            name="time_frequency",
            facet="variable",
            multiple=True,
            help=(
                "Set the time_frequency(s) of the input data. Will only have an"
                " effect if you haven't selected the <input_directory> "
                "parameter."
            ),
        ),
        SelectField(
            name="file_pattern",
            options=read_file_patters(),
            help="Select from predefined file patterns.",
            max_items=1000,
            mandatory=True,
        ),
    )

    def run_tool(self, config_dict):
        print(config_dict)
        # return self.prepare_output(config_dict["output_dir"])
